@@ -38,8 +38,10 @@
 #define FLAG_FORCE_SORT         0x00000800
 #define FLAG_CAPTION_RES        0x00001000
 #define FLAG_SKIP_DLNA_PN       0x00002000 /* during browsing */
+#define FLAG_CONVERT_MS         0x00004000 /* convert ms to s */
 /* Response-related flags */
-#define FLAG_HAS_CAPTIONS       0x80000000
+#define FLAG_HAS_CAPTIONS       0x10000000
+#define RESPONSE_TRUNCATED      0x80000000
 #define RESPONSE_FLAGS          0xF0000000
 
 enum match_types {
@@ -69,6 +71,7 @@ enum client_types {
 	ESamsungSeriesA,
 	ESamsungSeriesB,
 	ESamsungSeriesCDEBDP,
+	ESamsungSeriesQ,
 	ESamsungSeriesCDE,
 	ESamsungBDJ5500,
 	ESonyBDP,
