@@ -137,7 +137,7 @@ init_source(j_decompress_ptr cinfo)
 	return;
 }
 
-static int
+static boolean
 fill_input_buffer(j_decompress_ptr cinfo)
 {
 	struct my_src_mgr *src = (void *)cinfo->src;
@@ -190,7 +190,7 @@ jpeg_memory_src(j_decompress_ptr cinfo, const unsigned char * buffer, size_t buf
 	src->pub.bytes_in_buffer = bufsize;
 }
 
-jmp_buf setjmp_buffer;
+static jmp_buf setjmp_buffer;
 /* Don't exit on error like libjpeg likes to do */
 static void
 libjpeg_error_handler(j_common_ptr cinfo)
