@@ -96,7 +96,7 @@ dlna_timestamp_is_present(const char *filename, int *raw_packet_size)
 			if (buffer[i + MPEG_TS_PACKET_LENGTH_DLNA] == MPEG_TS_SYNC_CODE &&
 			    buffer[i + MPEG_TS_PACKET_LENGTH_DLNA*2] == MPEG_TS_SYNC_CODE)
 			{
-			        *raw_packet_size = MPEG_TS_PACKET_LENGTH_DLNA;
+				*raw_packet_size = MPEG_TS_PACKET_LENGTH_DLNA;
 				if (buffer[i+MPEG_TS_PACKET_LENGTH] == 0x00 &&
 				    buffer[i+MPEG_TS_PACKET_LENGTH+1] == 0x00 &&
 				    buffer[i+MPEG_TS_PACKET_LENGTH+2] == 0x00 &&
@@ -106,8 +106,8 @@ dlna_timestamp_is_present(const char *filename, int *raw_packet_size)
 					return 1;
 			} else if (buffer[i + MPEG_TS_PACKET_LENGTH] == MPEG_TS_SYNC_CODE &&
 				   buffer[i + MPEG_TS_PACKET_LENGTH*2] == MPEG_TS_SYNC_CODE) {
-			    *raw_packet_size = MPEG_TS_PACKET_LENGTH;
-			    return 0;
+				*raw_packet_size = MPEG_TS_PACKET_LENGTH;
+				return 0;
 			}
 		}
 	}
@@ -125,13 +125,13 @@ check_for_captions(const char *path, int64_t detailID)
 	strncpyt(file, path, sizeof(file));
 	p = strip_ext(file);
 	if (!p)
-		p = strrchr(file, '\0');
+		return;
 
 	/* If we weren't given a detail ID, look for one. */
 	if (!detailID)
 	{
 		detailID = sql_get_int64_field(db, "SELECT ID from DETAILS where (PATH > '%q.' and PATH <= '%q.z')"
-		                            " and MIME glob 'video/*' limit 1", file, file);
+						   " and MIME glob 'video/*' limit 1", file, file);
 		if (detailID <= 0)
 		{
 			//DPRINTF(E_MAXDEBUG, L_METADATA, "No file found for caption %s.\n", path);
@@ -156,7 +156,7 @@ check_for_captions(const char *path, int64_t detailID)
 	}
 }
 
-void
+static void
 parse_nfo(const char *path, metadata_t *m)
 {
 	FILE *nfo;
@@ -174,7 +174,7 @@ parse_nfo(const char *path, metadata_t *m)
 	}
 	DPRINTF(E_DEBUG, L_METADATA, "Parsing .nfo file: %s\n", path);
 	buf = calloc(1, file.st_size + 1);
-	if (buf)
+	if (!buf)
 		return;
 	nfo = fopen(path, "r");
 	if (!nfo)
@@ -182,7 +182,7 @@ parse_nfo(const char *path, metadata_t *m)
 		free(buf);
 		return;
 	}
-	nread = fread(&buf, 1, sizeof(buf), nfo);
+	nread = fread(buf, 1, file.st_size, nfo);
 	fclose(nfo);
 	
 	ParseNameValue(buf, nread, &xml, 0);
@@ -237,6 +237,14 @@ parse_nfo(const char *path, metadata_t *m)
 		free(esc_tag);
 	}
 
+	val = GetValueFromNameValueList(&xml, "season");
+	if (val)
+		m->disc = atoi(val);
+
+	val = GetValueFromNameValueList(&xml, "episode");
+	if (val)
+		m->track = atoi(val);
+
 	ClearNameValueList(&xml);
 	free(buf);
 }
@@ -287,7 +295,7 @@ GetFolderMetadata(const char *name, const char *path, const char *artist, const
 }
 
 int64_t
-GetAudioMetadata(const char *path, char *name)
+GetAudioMetadata(const char *path, const char *name)
 {
 	char type[4];
 	static char lang[6] = { '\0' };
@@ -303,7 +311,6 @@ GetAudioMetadata(const char *path, char *name)
 
 	if ( stat(path, &file) != 0 )
 		return 0;
-	strip_ext(name);
 
 	if( ends_with(path, ".mp3") )
 	{
@@ -346,6 +353,16 @@ GetAudioMetadata(const char *path, char *name)
 		strcpy(type, "pcm");
 		m.mime = strdup("audio/L16");
 	}
+	else if( ends_with(path, ".dsf") )
+	{
+		strcpy(type, "dsf");
+		m.mime = strdup("audio/x-dsd");
+	}
+	else if( ends_with(path, ".dff") )
+	{
+		strcpy(type, "dff");
+		m.mime = strdup("audio/x-dsd");
+	}
 	else
 	{
 		DPRINTF(E_WARN, L_METADATA, "Unhandled file extension on %s\n", path);
@@ -384,7 +401,9 @@ GetAudioMetadata(const char *path, char *name)
 	}
 	else
 	{
-		m.title = name;
+		free_flags |= FLAG_TITLE;
+		m.title = strdup(name);
+		strip_ext(m.title);
 	}
 	for( i = ROLE_START; i < N_ROLE; i++ )
 	{
@@ -483,7 +502,7 @@ GetAudioMetadata(const char *path, char *name)
 }
 
 /* For libjpeg error handling */
-jmp_buf setjmp_buffer;
+static jmp_buf setjmp_buffer;
 static void
 libjpeg_error_handler(j_common_ptr cinfo)
 {
@@ -493,7 +512,7 @@ libjpeg_error_handler(j_common_ptr cinfo)
 }
 
 int64_t
-GetImageMetadata(const char *path, char *name)
+GetImageMetadata(const char *path, const char *name)
 {
 	ExifData *ed;
 	ExifEntry *e = NULL;
@@ -514,7 +533,6 @@ GetImageMetadata(const char *path, char *name)
 	//DEBUG DPRINTF(E_DEBUG, L_METADATA, "Parsing %s...\n", path);
 	if ( stat(path, &file) != 0 )
 		return 0;
-	strip_ext(name);
 	//DEBUG DPRINTF(E_DEBUG, L_METADATA, " * size: %jd\n", file.st_size);
 
 	/* MIME hard-coded to JPEG for now, until we add PNG support */
@@ -639,13 +657,15 @@ no_exifdata:
 	else if( (width <= 4096 && height <= 4096) || !GETFLAG(DLNA_STRICT_MASK) )
 		m.dlna_pn = strdup("JPEG_LRG");
 	xasprintf(&m.resolution, "%dx%d", width, height);
+	m.title = strdup(name);
+	strip_ext(m.title);
 
 	ret = sql_exec(db, "INSERT into DETAILS"
 	                   " (PATH, TITLE, SIZE, TIMESTAMP, DATE, RESOLUTION,"
 	                    " ROTATION, THUMBNAIL, CREATOR, DLNA_PN, MIME) "
 	                   "VALUES"
 	                   " (%Q, '%q', %lld, %lld, %Q, %Q, %u, %d, %Q, %Q, %Q);",
-	                   path, name, (long long)file.st_size, (long long)file.st_mtime, m.date,
+	                   path, m.title, (long long)file.st_size, (long long)file.st_mtime, m.date,
 	                   m.resolution, m.rotation, thumb, m.creator, m.dlna_pn, m.mime);
 	if( ret != SQLITE_OK )
 	{
@@ -662,7 +682,7 @@ no_exifdata:
 }
 
 int64_t
-GetVideoMetadata(const char *path, char *name)
+GetVideoMetadata(const char *path, const char *name)
 {
 	struct stat file;
 	int ret, i;
@@ -685,7 +705,6 @@ GetVideoMetadata(const char *path, char *name)
 	//DEBUG DPRINTF(E_DEBUG, L_METADATA, "Parsing video %s...\n", name);
 	if ( stat(path, &file) != 0 )
 		return 0;
-	strip_ext(name);
 	//DEBUG DPRINTF(E_DEBUG, L_METADATA, " * size: %jd\n", file.st_size);
 
 	ret = lav_open(&ctx, path);
@@ -1498,10 +1517,8 @@ video_no_dlna:
 	if( ext )
 	{
 		strcpy(ext+1, "nfo");
-		if( access(nfo, F_OK) == 0 )
-		{
+		if( access(nfo, R_OK) == 0 )
 			parse_nfo(nfo, &m);
-		}
 	}
 
 	if( !m.mime )
@@ -1533,7 +1550,34 @@ video_no_dlna:
 	}
 
 	if( !m.title )
+	{
 		m.title = strdup(name);
+		strip_ext(m.title);
+	}
+
+	if (!m.disc && !m.track)
+	{
+		/* Search for Season and Episode in the filename */
+		char *p = (char*)name, *s;
+		while ((s = strpbrk(p, "Ss")))
+		{
+			unsigned season = strtoul(s+1, &p, 10);
+			unsigned episode = 0;
+			if (season > 0 && p)
+			{
+				while (isblank(*p) || ispunct(*p))
+					p++;
+				if (*p == 'E' || *p == 'e')
+					episode = strtoul(p+1, NULL, 10);
+			}
+			if (season && episode)
+			{
+				m.disc = season;
+				m.track = episode;
+			}
+			p = s + 1;
+		}
+	}
 
 	album_art = find_album_art(path, m.thumb_data, m.thumb_size);
 	freetags(&video);
@@ -1541,13 +1585,13 @@ video_no_dlna:
 
 	ret = sql_exec(db, "INSERT into DETAILS"
 	                   " (PATH, SIZE, TIMESTAMP, DURATION, DATE, CHANNELS, BITRATE, SAMPLERATE, RESOLUTION,"
-	                   "  TITLE, CREATOR, ARTIST, GENRE, COMMENT, DLNA_PN, MIME, ALBUM_ART) "
+	                   "  TITLE, CREATOR, ARTIST, GENRE, COMMENT, DLNA_PN, MIME, ALBUM_ART, DISC, TRACK) "
 	                   "VALUES"
-	                   " (%Q, %lld, %lld, %Q, %Q, %u, %u, %u, %Q, '%q', %Q, %Q, %Q, %Q, %Q, '%q', %lld);",
+	                   " (%Q, %lld, %lld, %Q, %Q, %u, %u, %u, %Q, '%q', %Q, %Q, %Q, %Q, %Q, '%q', %lld, %u, %u);",
 	                   path, (long long)file.st_size, (long long)file.st_mtime, m.duration,
 	                   m.date, m.channels, m.bitrate, m.frequency, m.resolution,
 	                   m.title, m.creator, m.artist, m.genre, m.comment, m.dlna_pn,
-	                   m.mime, album_art);
+	                   m.mime, album_art, m.disc, m.track);
 	if( ret != SQLITE_OK )
 	{
 		DPRINTF(E_ERROR, L_METADATA, "Error inserting details for '%s'!\n", path);
